MODULE OrdnerADT1 EXPORTS OrdnerADT;
(* Implementierung des ADT Ordner
Autor : Horst Lichter, RWTH Aachen
Umgebung : SRC-Modula-3 rel. 3.6, Windows NT 4.0
Erstellt : 16.08.98
Letzte Aenderung: 09.01.2001
*)



CONST MaxTexte = 20;	
TYPE Fassungsvermoegen = [1 .. MaxTexte];
TYPE Texte = ARRAY Fassungsvermoegen OF TEXT;

REVEAL Ordner = BRANDED REF RECORD			(* Datentyp wird aufgedeckt mit brandzeichen versehen *) 
                  ordnerInhalt : Texte;		(* Array das die texte aufnimmt *)
                  anzahlTexte : CARDINAL;		(* Zaehler der angibt wieviele Texte schon abgelegt wurden *)
                  beschriftung : TEXT := "";	(* Titel des Ordners *)
                END;


PROCEDURE LegeTextAb (VAR o: Ordner; t : TEXT)=
(* Legt einen text t in den Ordner o ab *)
BEGIN
  o^.anzahlTexte := o^.anzahlTexte + 1;	(* erhhen des Zaehlers *)
  o^.ordnerInhalt[o^.anzahlTexte] := t;	(* Text wird im naechsten freien Feld abgelegt *)
END LegeTextAb;


PROCEDURE EntnehmeText (VAR o: Ordner) : TEXT =
(* Entnimmt dem Ordner o den letzten hinzugefuegten Text *)
VAR t : TEXT;
BEGIN
  t := o^.ordnerInhalt[o^.anzahlTexte];		(* Text aus dem letzten Feld des Arrays wird gelesen *)
  o^.ordnerInhalt[o^.anzahlTexte] := "";		(* Das Feld wird geloescht *)
  o^.anzahlTexte := o^.anzahlTexte - 1;		(* Der Zaehler wird um ein verringert *)
  RETURN t;							(* Der Text wird zurck gegeben *)
END EntnehmeText;


PROCEDURE IstVoll (o : Ordner) : BOOLEAN =
(* Ist Ordner o voll wird TRUE zurck gegeben sonst FALSE *)
BEGIN
  RETURN (o^.anzahlTexte = MaxTexte);		(* Vergleich des Zaehlers mit MaxTexte *)
END IstVoll;

PROCEDURE IstLeer (o : Ordner) : BOOLEAN =
(* Ist der ordner o leer wird TRUE zurckgegeben sonst FALSE *)
BEGIN
  RETURN (o^.anzahlTexte = 0);			(* Vergleich des Zaehlers mit 0 *)
END IstLeer;


PROCEDURE Beschrifte (VAR o: Ordner; t : TEXT)=
(* legt den text t als beschriftung des Ordners o fest *)
BEGIN
  o^.beschriftung := t;					(* Die Variable "beschriftung" bekommt den ubergebenen wert zugewiesen *)
END Beschrifte;


PROCEDURE GibBeschriftung (o : Ordner) : TEXT =
(* Gibt die Beschriftung des ordners o zurck *)
BEGIN
  RETURN o^.beschriftung;				
END GibBeschriftung;


PROCEDURE Anlegen () : Ordner =
(* Erstellt einen neuen Ordner und gibt diesen zurck *)
VAR o : Ordner;			
BEGIN
  o := NEW(Ordner);		(* Neuer Ordner wird erzeugt und o zugewiesen *)	
  FOR i := FIRST(Fassungsvermoegen) TO LAST(Fassungsvermoegen) DO	
    o^.ordnerInhalt[i] := "";	(* alle felder werden auf "" gesetzt *)
  END;				
  o^.anzahlTexte := 0;		(* Keine Texte vorhanden, daher anzahlTexte = 0 *)
  RETURN o;				(* Der leere Ordner wird zurueckgegeben *)
END Anlegen;

BEGIN
END OrdnerADT1.